;
;  PCE CDROM access routines' sources
;

	.include "cdrom.inc"

	.bss

cdplay_end_ctl	.ds  1 ; saved 'cd_play end' information - type
cdplay_end_h	.ds  1 ; high byte of address
cdplay_end_m	.ds  1 ; mid byte
cdplay_end_l	.ds  1 ; low byte
cd_buf          .ds  4 ; Return buffer from some BIOS commands

	.code

;
; cd_reset(void)
; ----
; Reset CDROM
; ----
;
_cd_reset:	jsr	cd_reset
		rts

;
; cd_pause(void)
; ----
; Pause CDROM drive
; ----
;
_cd_pause:	jsr	cd_pause
		tax
		cla
		rts


;
; cd_unpause(void)
; ----
; Continue playing CDROM audio after pause
; ----
;
_cd_unpause:	lda	cdplay_end_ctl
		sta	<_dh
		lda	cdplay_end_h
		sta	<_cl
		lda	cdplay_end_m
		sta	<_ch
		lda	cdplay_end_l
		sta	<_dl
		lda	#CD_CURRPOS
		sta	<_bh
		jsr	cd_play
		tax
		cla
		rts

;
; cd_fade(char type)
;
;   type = $00 -> cancel fade
;          $08 -> PCM fadeout 6 seconds
;          $0A -> ADPCM fadeout 6 seconds
;          $0C -> PCM fadeout 2.5 seconds
;          $0E -> ADPCM fadeout 2.5 seconds
; ----
; Fade-out PCM/ADPCM audio
; ----
;
_cd_fade:	txa
		jsr	cd_fade
		tax
		cla
		rts


;
; cd_playtrk(int start_track, int end_track, int mode)
;   mode = CDPLAY_MUTE / CDPLAY_REPEAT / CDPLAY_NORMAL
; ----
; Play CDROM audio track
; ----
;
_cd_playtrk:	txa
		and	#CD_PLAYMODE
		ora	#CD_TRACK
		sta	<_dh		; end type + play mode
		sta	cdplay_end_ctl
		lda	#CD_TRACK
		sta	<_bh		; start type
		
	        lda	[__sp]		; track #
		bne	.endtrk

.endofdisc:	lda	<_dh		; repeat to end of disc
		ora	#CD_LEADOUT
		sta	<_dh
		sta	cdplay_end_ctl
		bra	.starttrk

.endtrk:	jsr	ex_binbcd
		sta	<_cl		; end track
		sta	cdplay_end_h
		stz	<_ch
		stz	cdplay_end_m
		stz	<_dl
		stz	cdplay_end_l

.starttrk:	ldy	#2
	        lda	[__sp],Y	; track #
		jsr	ex_binbcd
		sta	<_al		; from track
		stz	<_ah
		stz	<_bl
		addw	#4,<__sp
		jsr	cd_play
		tax
		cla
		rts


;
; cd_playmsf(int start_minute, int start_second, int start_frame,
;            int end_minute, int end_second, int end_frame, int mode)
;   mode = CDPLAY_MUTE / CDPLAY_REPEAT / CDPLAY_NORMAL
; ----
; Play CDROM from/to 'minute/second/frame'
; ----
;
_cd_playmsf:	txa
		and	#CD_PLAYMODE
		ora	#CD_MSF
		sta	<_dh		; end type + play mode
		sta	cdplay_end_ctl
		lda	#CD_MSF
		sta	<_bh		; start type
		
.endmsf:	ldy	#2
	        lda	[__sp]		; end frame
		jsr	ex_binbcd
		sta	<_dl
		sta	cdplay_end_l

	        lda	[__sp],Y	; end second
		jsr	ex_binbcd
		sta	<_ch
		sta	cdplay_end_m

		iny
		iny
	        lda	[__sp],Y	; end minute
		jsr	ex_binbcd
		sta	<_cl
		sta	cdplay_end_h

.startmsf:
		iny
		iny
	        lda	[__sp],Y	; start frame
		jsr	ex_binbcd
		sta	<_bl

		iny
		iny
	        lda	[__sp],Y	; start second
		jsr	ex_binbcd
		sta	<_ah

		iny
		iny
	        lda	[__sp],Y	; start minute
		jsr	ex_binbcd
		sta	<_al

		addw	#4,<__sp
		jsr	cd_play
		tax
		cla
		rts

;
; char cd_numtrk(void)
; ----
; return number of tracks on CDROM
; ----
;
_cd_numtrk:	stw	#cd_buf,<_bx
		stz	<_al		; request type 0
		jsr	cd_dinfo
		cmp	#$00
		bne	.err
		lda	cd_buf+1
		jsr	ex_bcdbin
		clx
		sax
		rts
.err:		lda	#$ff
		tax
		rts

;
; char cd_trktype(char track)
; ----
; Return type of track (data/audio)
; ----
;
_cd_trktype:	sax
		jsr	ex_binbcd
		sta	<_ah		; track #
		stw	#cd_buf,<_bx
		lda	#2
		sta	<_al		; request type 2
		jsr	cd_dinfo
		cmp	#$00
		bne	.err
		ldx	cd_buf+3	; track type
		cla
		rts
.err:		lda	#$ff
		tax
		rts


;
; ad_reset(void)
; ----
; Reset ADPCM device
; ----
;
_ad_reset:	jsr	ad_reset
		rts

;--------------------
; NOT IMPLEMENTED YET
;--------------------
;
;
; cd_seek()
; ----
; Seek CDROM drive
; ----
;
;_cd_seek:
;		rts
;
;
; cd_read()
; ----
; Read from CDROM
; ----
;
;_cd_read:
;		rts
;
;
; cd_stat()
; ----
; Get CDROM status
; ----
;
;_cd_stat:
;		rts
;
;
;--------------------


;
; int cd_getver()
; ----
; get CDROM system card version number
; MSB = major number
; LSB = minor number
; ----
;
_cd_getver:
		jsr	ex_getver
		tya
		sax
		rts


;
; int ac_exists(void)
; ----
; Detect Arcade Card (return 1 if true)
; ----
;
_ac_exists:	lda	ac_identflag
		ldx	#1
		cmp	#AC_IDENT
		beq	.true
		clx
.true:		cla
		rts

;
; grp_init(void)
; ----
; Initialize graphics context to use graphics functions
; ----
;
_grp_init:	lda	#VI_GINIT
		sta	<_dh
		stw	#$1000,<_ax		; graphic start address
		stw	#$3000,<_bx		; paint stack
		jsr	grp_bios
;		lda	#VI_CASHCLR
;		sta	<_dh
;		jsr	grp_bios
		rts

;
; grp_cls(void)
; ----
; Clear screen in graphics
; ----
;
_grp_cls:	lda	#VI_CLS
		sta	<_dh
		jsr	grp_bios
		rts

;
; grp_set(x, y, color)
; ----
; Set a point at (x,y) to be color (0-15)
; ----
;
_grp_set:	lda	#VI_PSET
		sta	<_dh
		stx	<_dl		; color
		lda	[__sp]		; y-coord
		sta	<_al
		ldy	#2
		lda	[__sp],Y	; x-coord
		sta	<_ah
		jsr	grp_bios
		addw	#4,__sp
		rts

;
; int grp_point(x, y)
; ----
; Find out the color of a point at (x, y)
; ----
;
_grp_point:	lda	#VI_POINT
		sta	<_dh
		stx	<_al		; y-coord
		lda	[__sp]
		sta	<_ah		; x-coord
		jsr	grp_bios
		sax
		addw	#2,__sp
		cla
		rts

;
; grp_line(x1, y1, x2, y2, color)
; ----
; draw a line
; ----
;
_grp_line:	lda	#VI_LINE
		sta	<_dh
		stx	<_dl		; color
		lda	[__sp]
		sta	<_bl		; end y-coord
		ldy	#2
		lda	[__sp],Y
		sta	<_bh		; end x-coord
		ldy	#4
		lda	[__sp],Y
		sta	<_al		; start y-coord
		lda	#6
		lda	[__sp],Y
		sta	<_ah		; start x-coord
		jsr	grp_bios
		addw	#8,__sp
		rts

;
; grp_box(x1, y1, x2, y2, color, fill)
; ----
; Draw a box
; ----
;
_grp_box:	sax
		cmp	#GRP_FILL
		beq	.fill
		lda	#VI_BOX
		bra	.l1
.fill:		lda	#VI_BOXF
.l1:		sta	<_dh
		lda	[__sp]
		sta	<_dl		; color
		ldy	#2
		lda	[__sp],Y
		sta	<_bl		; end y-coord
		ldy	#4
		lda	[__sp],Y
		sta	<_bh		; end x-coord
		ldy	#6
		lda	[__sp],Y
		sta	<_al		; start y-coord
		ldy	#8
		lda	[__sp],Y
		sta	<_ah		; start x-coord
		jsr	grp_bios
		addw	#10,__sp
		rts

;
; grp_flood(x, y, color)
; ----
; flood-fill with color starting at (x, y)
; ----
;
_grp_flood:	lda	#VI_FLOOD
		sta	<_dh
		stx	<_dl		; color
		lda	[__sp]
		sta	<_al		; y-coord
		ldy	#2
		lda	[__sp],Y
		sta	<_ah
		jsr	grp_bios
		addw	#4,__sp
		rts

;
; grp_paint(x, y, ?, color)
; ----
; Paint (not sure how or why)
; ----
;
_grp_paint:	lda	#VI_PAINT
		sta	<_dh
		stx	<_dl		; color
		lda	[__sp]
		sta	<_bl		; ? (low bit)
		ldy	#1
		lda	[__sp],Y
		sta	<_bh		; ? (high bit)
		iny
		lda	[__sp],Y
		sta	<_al		; y-coord
		iny
		iny
		lda	[__sp],Y
		sta	<_ah		; x-coord
		jsr	grp_bios
		addw	#6,__sp
		rts

